# _*_ coding: utf-8 _*_
#
# hspylib-firebase v0.9.74
#
# Package: main.firebase
"""Package initialization."""

__all__ = [
  '__classpath__', 
  '__main__', 
  'core', 
  'entity', 
  'exception'
]
__version__ = '0.9.74'
